unit DiffUnit;

interface

uses
  Windows,
  classes,
  sysutils,
  UVersieIn,
  diffmake;

const
  CDIFF_ISDELTAFILE = 'MKDiff Delta File'+ #0;
  CDIFF_ISDELTAFILELEN = 18;
  CDIFF_VERION_HIGH = 1;
  CDIFF_VERION_LOW = 0;

  CDIFF_ERROR_OK          =   0 ;// 'OK'
  CDIFF_ERROR_GENERAL     =  -1 ;// 'General error'
  CDIFF_ERROR_NEW_NOFI    =  -2 ;// 'Newer file doesn''t have File information');
  CDIFF_ERROR_OLD_NOFI    =  -3 ;// 'Old file doesn''t have File information');
  CDIFF_ERROR_CRC         =  -4 ;// 'CRC is wrong');
  CDIFF_ERROR_SOURCE_FI   =  -5 ;// 'Wrong version of source file!');
  CDIFF_ERROR_SOURCE_NOFI =  -6 ;// 'Source File has no File Information included!');
  CDIFF_ERROR_NO_EXP_FILE =  -7 ;// 'There''s no file to Expand!');
  CDIFF_ERROR_DF_VERSION  =  -8 ;// 'Wrong version of delta file!' + #13#10 + 'Check if newer version is available');
  CDIFF_ERROR_NO_DF       =  -9 ;// 'This no delta file of MKDiff!');
  CDIFF_ERROR_ALREADY_UPD = -10 ;// 'Source file already updated!');

type
  TFileDiffVersion = record
    RecogString: string[CDIFF_ISDELTAFILELEN];
    High: integer;
    Low: integer;
  end;
  PFileDiffInfo = ^TFileDiffInfo;
  TFileDiffInfo = record
    RecVersion: integer;
    FileName: string[70];
    dwFileVersionMSH,
    dwFileVersionMSL,
    dwFileVersionLSH,
    dwFileVersionLSL: DWORD;
    dwNewFileVersionMSH,
    dwNewFileVersionMSL,
    dwNewFileVersionLSH,
    dwNewFileVersionLSL: DWORD;
    FileCRC: Cardinal;
  end;

procedure MainProc;

implementation

uses
  CmnFunc2;

function CalcFileCRC(strFile: string): Cardinal;
var
  buffer: string;
  Stream: TStream;
begin
  Stream := TFileStream.Create(strFile, fmOpenRead + fmShareDenyNone);
  try
    buffer := StringOfChar(#32, Stream.Size);
    Stream.ReadBuffer(Pointer(buffer)^, Stream.Size);
  finally
    Stream.Free;
  end;
  Result := CalculateCRC32(Pointer(buffer)^, Length(buffer));
  Setlength(buffer, 0);
end;

function CreateMKDiffFile(strFileOrg, strFileNew: string): integer;
var
  FileDiffInfo: TFileDiffInfo;
  FileDiffVersion: TFileDiffVersion;
  DeltaStream: TMemoryStream;
  InStream: TFileStream;
  UseStream: TFileStream;
  OutStream: TFileStream;
  strDeltaFile: string;
begin
  Result := -1; // 'General error'
  FileDiffVersion.High := CDIFF_VERION_HIGH;
  FileDiffVersion.Low := CDIFF_VERION_LOW;
  FileDiffVersion.RecogString := CDIFF_ISDELTAFILE;
  FileDiffInfo.FileName := strFileOrg;
  FileDiffInfo.RecVersion := 1;
  FileDiffInfo.FileCRC := CalcFileCRC(strFileOrg);
  strDeltaFile := strFileOrg + '_' + GetFileversionNumber(strFileNew) + '.mkd';
  if RetrieveFileVersion(strFileOrg,
                         FileDiffInfo.dwFileVersionMSH, FileDiffInfo.dwFileVersionMSL,
                         FileDiffInfo.dwFileVersionLSH, FileDiffInfo.dwFileVersionLSL) then
  begin
    if RetrieveFileVersion(strFileNew,
                         FileDiffInfo.dwNewFileVersionMSH, FileDiffInfo.dwNewFileVersionMSL,
                         FileDiffInfo.dwNewFileVersionLSH, FileDiffInfo.dwNewFileVersionLSL) then
    begin
      try
        DeltaStream := TMemoryStream.Create;
        try
          DeltaStream.WriteBuffer(FileDiffVersion, Sizeof(TFileDiffVersion));
          DeltaStream.WriteBuffer(FileDiffInfo, Sizeof(TFileDiffInfo));

          InStream  := TFileStream.Create(strFileNew, fmopenread + fmShareDenyNone);
          UseStream := TFileStream.Create(strFileOrg, fmopenread + fmShareDenyNone);
          try
            DiffStreamCompress(InStream,UseStream, DeltaStream, 25)
          finally
            InStream.Free;
            UseStream.Free;
          end;
          OutStream := TFileStream.Create(strDeltaFile, fmcreate);
          try
{$IFDEF NOCONSOLE}
            Writeln('Started...');
{$ENDIF}
            DeltaStream.Position := 0;
            OutStream.CopyFrom(DeltaStream, DeltaStream.Size);
{$IFDEF NOCONSOLE}
            Writeln('Ready...');
{$ENDIF}
            Result := CDIFF_ERROR_OK;
          finally
            OutStream.Free;
          end;
        finally
          DeltaStream.Free;
        end;
      except
{$IFDEF NOCONSOLE}
        on e: Exception do Writeln(e.Message);
{$ELSE}
        raise;
{$ENDIF}
      end;
    end else Result := CDIFF_ERROR_NEW_NOFI;
  end else Result := CDIFF_ERROR_OLD_NOFI;
end;

function ExpandMKDiffFile(strDeltaFile: string): integer;
var
  FileDiffInfo: TFileDiffInfo;
  FileDiffVersion: TFileDiffVersion;
  DeltaStream: TMemoryStream;
  InStream: TFileStream;
  UseStream: TFileStream;
  OutStream: TFileStream;
  strFileOrg: string;
  strFileNew: string;
  dwFileVersionMSH,
  dwFileVersionMSL,
  dwFileVersionLSH,
  dwFileVersionLSL: DWORD;
begin
  Result := CDIFF_ERROR_GENERAL;
  DeltaStream := TMemoryStream.Create;
  try
    DeltaStream.LoadFromFile(strDeltaFile);
    DeltaStream.ReadBuffer(FileDiffVersion, Sizeof(TFileDiffVersion));
    if FileDiffVersion.RecogString = CDIFF_ISDELTAFILE then
    begin
      if (FileDiffVersion.High = CDIFF_VERION_HIGH) and
         (FileDiffVersion.Low = CDIFF_VERION_LOW) then
      begin
        DeltaStream.ReadBuffer(FileDiffInfo, Sizeof(TFileDiffInfo));
        strFileOrg := AddBackSlash(ExtractFilePath(strDeltaFile)) + FileDiffInfo.FileName;
        if FileExists(strFileOrg) then begin
          if RetrieveFileVersion(strFileOrg, dwFileVersionMSH, dwFileVersionMSL,
                                 dwFileVersionLSH, dwFileVersionLSL) then
          begin
            if (dwFileVersionMSH = FileDiffInfo.dwNewFileVersionMSH) and
               (dwFileVersionMSL = FileDiffInfo.dwNewFileVersionMSL) and
               (dwFileVersionLSH = FileDiffInfo.dwNewFileVersionLSH) and
               (dwFileVersionLSL = FileDiffInfo.dwNewFileVersionLSL) then
            begin
              Result := CDIFF_ERROR_ALREADY_UPD;
            end else
              if (dwFileVersionMSH = FileDiffInfo.dwFileVersionMSH) and
                 (dwFileVersionMSL = FileDiffInfo.dwFileVersionMSL) and
                 (dwFileVersionLSH = FileDiffInfo.dwFileVersionLSH) and
                 (dwFileVersionLSL = FileDiffInfo.dwFileVersionLSL) then
              begin
                if FileDiffInfo.FileCRC = CalcFileCRC(strFileOrg) then
                begin
                  try
                    strFileNew := strFileOrg + '.new';
                    if FileExists(strFileNew) then
                      DeleteFile(strFileNew);
                    OutStream := TFileStream.Create(strFileNew, fmcreate);
                    try
                      InStream  := TFileStream.Create(strFileOrg, fmopenread + fmShareDenyNone);
                      try
{$IFDEF NOCONSOLE}
                        Writeln('Started...');
{$ENDIF}
                        DiffStreamExtract(DeltaStream, InStream, OutStream);

                        CopyFile(PChar(strFileOrg), PChar(strFileOrg + '_' + GetFileversionNumber(strFileOrg) + '.org'), True);
{$IFDEF NOCONSOLE}
                        Writeln('Ready...');
{$ENDIF}
                        Result := CDIFF_ERROR_OK;
                      finally
                        InStream.Free;
                      end;
                    finally
                      OutStream.Free;
                    end;
                    CopyFile(PChar(strFileNew), PChar(strFileOrg), False);
                    if FileExists(strFileNew) then
                      DeleteFile(strFileNew);
                  except
{$IFDEF NOCONSOLE}
                    on e: Exception do Writeln(e.Message);
{$ELSE}
                    raise;
{$ENDIF}
                  end;
                end else Result := CDIFF_ERROR_CRC;
              end else Result := CDIFF_ERROR_SOURCE_FI;
          end else Result := CDIFF_ERROR_SOURCE_NOFI;
        end else Result := CDIFF_ERROR_NO_EXP_FILE;
      end else Result := CDIFF_ERROR_DF_VERSION;
    end else Result := CDIFF_ERROR_NO_DF;
  finally
    DeltaStream.Free;
  end;
end;

{---------------------------------------}

procedure ShowHelp;
begin
  WriteLn('To create difference file:');
  WriteLn;
  WriteLn('MKDiff <Original file> <newer file>');
  WriteLn;
  WriteLn;
  WriteLn('To use difference file:');
  WriteLn;
  WriteLn('MKDiff <delta file>');
  WriteLn;
  WriteLn;
  WriteLn;
  Writeln('Press ENTER');
  Readln;
end;

{---------------------------------------}

procedure MainProc;
var
  orgFile, newFile, deltaFile: string;
  tmpCurrentDir: string;
  ErrorCode: integer;
begin
  WriteLn('MKDiff');
  WriteLn('Copyright (c) 1999 M.C. van der Kooij');
  WriteLn;
  WriteLn('Creates difference files for executables and dll''s');
  WriteLn;
  WriteLn('Based on:');
  WriteLn('Difference Maker');
  WriteLn('Copyright (c) 1997,1998 S.Kurinny & S.Kostinsky');
  WriteLn;
  WriteLn;
  If (ParamCount = 0) or (ParamCount > 2) then
  begin
    ShowHelp;
    exit;
  end;
  ErrorCode := 1;
  tmpCurrentDir := GetCurrentDir;
  try
    if (ParamCount = 2) then begin
      orgFile := ParamStr(1);
      newFile := ParamStr(2);
      SetCurrentDir(ExtractFilePath(orgFile));
      orgFile := ExtractFileName(orgFile);
      newFile := ExtractFileName(newFile);
      if not FileExists(orgFile) then
        WriteLn('Original file does not exists!')
      else
        if not FileExists(newFile) then
          WriteLn('Newer file does not exists!')
        else
          ErrorCode := CreateMKDiffFile(orgFile, newFile);
    end else begin
      deltaFile := ParamStr(1);
      if not FileExists(deltaFile) then
        WriteLn('Delta file does not exists!')
      else begin
        SetCurrentDir(ExtractFilePath(deltaFile));
        ErrorCode := ExpandMKDiffFile(ExtractFileName(deltaFile));
      end;
    end;
    case ErrorCode of
      CDIFF_ERROR_OK          : WriteLn('OK');
      CDIFF_ERROR_GENERAL     : WriteLn('General error');
      CDIFF_ERROR_NEW_NOFI    : WriteLn('Newer file doesn''t have File information');
      CDIFF_ERROR_OLD_NOFI    : WriteLn('Old file doesn''t have File information');
      CDIFF_ERROR_CRC         : WriteLn('CRC is wrong');
      CDIFF_ERROR_SOURCE_FI   : WriteLn('Wrong version of source file!');
      CDIFF_ERROR_SOURCE_NOFI : WriteLn('Source File has no File Information included!');
      CDIFF_ERROR_NO_EXP_FILE : WriteLn('There''s no file to Expand!');
      CDIFF_ERROR_DF_VERSION  : WriteLn('Wrong version of delta file!' + #13#10 + 'Check if newer version is available');
      CDIFF_ERROR_NO_DF       : WriteLn('This no delta file of MKDiff!');
      CDIFF_ERROR_ALREADY_UPD : WriteLn('Source file already updated!');
    end;
  finally
    SetCurrentDir(tmpCurrentDir);
  end;
end;

begin

end.