unit UVersieIn;

{ Unit UVersieIn

Bevat een routine voor het uitlezen van versie informatie zoals dat in een
executable of dll wordt gestopt.

Tevens bevat het een component TVersionInfo met de volgende properties:
       FileName  Naam van Exe of dll waarvan Versie informatie moet worden
                 bekeken
       Info      Stringlist met versie informatie, zie hieronder voor de
                 omschrijving van de mogelijke Values in de Stringlist

Er wordt ook een property editor geinstalleerd voor de property FileName.

De informatie wordt in een StringList geplaatst en kan uitgelezen worden mbv:
TStringList.Values['...']

De Values die opgevraagd kunnen worden zijn:

   Signature
   StrucVersion
   FileVersion             <FileVersionMS> + <FileVersionLS>
   FileVersionMS
   FileVersionLS
   ProductVersion          <ProductVersionMS> + <ProductVersionLS>
   ProductVersionMS
   ProductVersionLS
   dwFileFlagsMask
   dwFileFlags
   dwFileOS
   dwFileType
   dwFileSubType
   dwFileDateMS
   dwFileDateLS
   Localization            De 8 cijferige code met de codepage en taal code
   L_CompanyName
   L_FileDescription
   L_FileVersion
   L_InternalName
   L_LegalCopyright
   L_LegalTradeMarks
   L_OriginalFileName
   L_ProductName
   L_ProductVersion
   L_Comments

Zie GetFileVersionInfo voor verdere uitleg van deze waarden

}
interface

uses
  Windows, Classes, SysUtils
{$IFNDEF CONSOLE}
  ,DsgnIntf,  Dialogs, Forms
{$ENDIF}
  ;

type
  TVersionInfo=class(TComponent)
  private
    FFile: String;
    FVersionInfo: TStringList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetInfo: TStringList;
  published
    property FileName: String read FFile write FFile;
    property Info: TStringList read GetInfo;
  end;

{$IFNDEF CONSOLE}
  TMKFileNameProperty = class( TStringProperty )
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;
{$ENDIF}
  TTimeOfWhat = (ftCreationTime, ftLastAccessTime, ftLastWriteTime);
  TFileTimeComparision = (ftError, ftFileOneIsOlder, ftFileTimesAreEqual,
                          ftFileTwoIsOlder);

{$IFNDEF CONSOLE}
procedure Register;
{$ENDIF}

function StabCompareFileTime(const FileNameOne, FileNameTwo: String;
                             ComparisonType: TTimeOfWhat): TFileTimeComparision;
function StabGetFileTimeToDateTime(FileTime: TFileTime): TDateTime;
function StabGetFileTime(const FileName: String; ComparisonType: TTimeOfWhat): TFileTime;

function GetFileversionNumber(strFile: string): string;
function RetrieveVersionInfo(strFile: string; lstFileInfo: TStringList): boolean;
function RetrieveFileVersion(strFile: string;
                             var dwFileVersionMSH, dwFileVersionMSL,
                                 dwFileVersionLSH, dwFileVersionLSL: DWORD): boolean;

implementation

{$IFNDEF CONSOLE}
procedure Register;
begin
  RegisterComponents('Stabiplan vcl', [TVersionInfo]);
  RegisterPropertyEditor( TypeInfo( string ), TVersionInfo, 'FileName',
                          TMKFileNameProperty );
end;
{$ENDIF}

function TVersionInfo.GetInfo: TStringList;
begin
  FVersionInfo.Clear;
  if FileExists(FFile) then
  begin
    RetrieveVersionInfo(FFile, FVersionInfo);
  end;
  Result := FVersionInfo;
end;

constructor TVersionInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FVersionInfo := TStringList.Create;
end;

destructor TVersionInfo.Destroy;
begin
  FVersionInfo.Free;
  inherited Destroy;
end;

{$IFNDEF CONSOLE}
{=================================}
{== TMKFileNameProperty Methods ==}
{=================================}

procedure TMKFileNameProperty.Edit;
var
  DlgOpen: TOpenDialog;
begin
  DlgOpen := TOpenDialog.Create( Application );
  DlgOpen.FileName := GetValue;
  DlgOpen.Filter := 'Executables|*.EXE;*.BAT;*.COM;*.PIF|All Files|*.*';
  DlgOpen.Options := DlgOpen.Options + [ofPathMustExist, ofFileMustExist];
  try
    if DlgOpen.Execute then
      SetValue( DlgOpen.FileName );
  finally
    DlgOpen.Free;
  end;
end;


function TMKFileNameProperty.GetAttributes: TPropertyAttributes;
begin
  {$IFDEF WIN32}
  Result := [ paDialog, paRevertable ];
  {$ELSE}
  Result := [ paDialog ];
  {$ENDIF}
end;
{$ENDIF}

function HighPart(intTotal: integer): integer;
begin
 result := intTotal shr 16;
end;

function LowPart(intTotal: integer): integer;
begin
  Result := intTotal and $FFFF;
end;

function IntToVersion(intTotal: integer): string;
begin
  Result := IntToStr(HighPart(intTotal)) + '.' + IntToStr(LowPart(intTotal));
end;

function RetrieveFileVersion(strFile: string;
                             var dwFileVersionMSH, dwFileVersionMSL,
                                 dwFileVersionLSH, dwFileVersionLSL: DWORD): boolean;
const
  InfoNum = 10;
  InfoStr : array [1..InfoNum] of String =
    ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName',
     'LegalCopyright', 'LegalTradeMarks', 'OriginalFilename',
     'ProductName', 'ProductVersion', 'Comments');

var
  intBufSize,
  hDummy, buf2len, len, i, intLocalization: DWORD;
  buffer, buf2: pointer;
  buffer3: array[0 .. 255] of CHAR;
  FixedFileInfo: TVSFIXEDFILEINFO;
  strLocalization: string;
  Value: PChar;

begin
  Result := False;
  intBufSize := GetFileVersionInfoSize(PCHAR(strFile), hDummy);
  if intBufSize = 0 then exit;
  GetMem(buffer, intBufSize);
  try
    if GetFileVersionInfo(PCHAR(strFile), hDummy, intBufSize, buffer) then
    begin
      if VerQueryValue(buffer, '\', buf2, buf2len) then
      begin
        FillChar(FixedFileInfo, buf2len, 0);
        Move(buf2^, FixedFileInfo, buf2len);
        dwFileVersionMSH := HighPart(FixedFileInfo.dwFileVersionMS);
        dwFileVersionMSL := LowPart(FixedFileInfo.dwFileVersionMS);
        dwFileVersionLSH := HighPart(FixedFileInfo.dwFileVersionLS);
        dwFileVersionLSL := LowPart(FixedFileInfo.dwFileVersionLS);
{        dwProductVersionMS := FixedFileInfo.dwProductVersionMS;
        dwProductVersionLS := FixedFileInfo.dwProductVersionLS;
        dwProductVersionMS := FixedFileInfo.dwProductVersionMS;
        dwProductVersionLS := FixedFileInfo.dwProductVersionLS;
        dwFileFlagsMask := FixedFileInfo.dwFileFlagsMask;

        dwFileFlags := FixedFileInfo.dwFileFlags;
        dwFileOS := FixedFileInfo.dwFileOS;
        dwFileType := FixedFileInfo.dwFileType;
        dwFileSubtype := FixedFileInfo.dwFileSubtype;
        dwFileDateMS := FixedFileInfo.dwFileDateMS;
        dwFileDateLS := FixedFileInfo.dwFileDateLS;}
        Result := true;
      end;
    end;
  finally
    FreeMem(buffer);
  end;
end;

function GetFileversionNumber(strFile: string): string;
var
  list: TStringList;
begin
  list := TStringList.Create;
  try
    RetrieveVersionInfo(strFile, list);
    Result := list.Values['L_FileVersion'];
  finally
    list.Free;
  end;
end;

function RetrieveVersionInfo(strFile: string; lstFileInfo: TStringList): boolean;
const
  InfoNum = 10;
  InfoStr : array [1..InfoNum] of String =
    ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName',
     'LegalCopyright', 'LegalTradeMarks', 'OriginalFilename',
     'ProductName', 'ProductVersion', 'Comments');

var
  intBufSize,
  hDummy, buf2len, len, i, intLocalization: DWORD;
  buffer, buf2: pointer;
  buffer3: array[0 .. 255] of CHAR;
  FixedFileInfo: TVSFIXEDFILEINFO;
  strLocalization: string;
  Value: PChar;

begin
  intBufSize := GetFileVersionInfoSize(PCHAR(strFile), hDummy);
  if intBufSize = 0 then exit;
  GetMem(buffer, intBufSize);
  try
    if GetFileVersionInfo(PCHAR(strFile), hDummy, intBufSize, buffer) then
    begin
      if VerQueryValue(buffer, '\', buf2, buf2len) then
      begin
        FillChar(FixedFileInfo, buf2len, 0);
        Move(buf2^, FixedFileInfo, buf2len);
        lstFileInfo.Add('Signature=' + IntToStr(FixedFileInfo.dwSignature));
        lstFileInfo.Add('StrucVersion=' + IntToVersion(FixedFileInfo.dwStrucVersion));
        lstFileInfo.Add('FileVersion=' + IntToVersion(FixedFileInfo.dwFileVersionMS) + '.' +
                                              IntToVersion(FixedFileInfo.dwFileVersionLS));
        lstFileInfo.Add('FileVersionMS=' + IntToVersion(FixedFileInfo.dwFileVersionMS));
        lstFileInfo.Add('FileVersionLS=' + IntToVersion(FixedFileInfo.dwFileVersionLS));
        lstFileInfo.Add('ProductVersion=' + IntToVersion(FixedFileInfo.dwProductVersionMS) + '.' +
                                                 IntToVersion(FixedFileInfo.dwProductVersionLS));
        lstFileInfo.Add('ProductVersionMS=' + IntToVersion(FixedFileInfo.dwProductVersionMS));
        lstFileInfo.Add('ProductVersionLS=' + IntToVersion(FixedFileInfo.dwProductVersionLS));
        lstFileInfo.Add('dwFileFlagsMask=' + IntToStr(FixedFileInfo.dwFileFlagsMask));

        lstFileInfo.Add('dwFileFlags=' + IntToStr(FixedFileInfo.dwFileFlags));
        lstFileInfo.Add('dwFileOS=' + IntToStr(FixedFileInfo.dwFileOS));
        lstFileInfo.Add('dwFileType=' + IntToStr(FixedFileInfo.dwFileType));
        lstFileInfo.Add('dwFileSubtype=' + IntToStr(FixedFileInfo.dwFileSubtype));
        lstFileInfo.Add('dwFileDateMS=' + IntToStr(FixedFileInfo.dwFileDateMS));
        lstFileInfo.Add('dwFileDateLS=' + IntToStr(FixedFileInfo.dwFileDateLS));
      end;
      if VerQueryValue(Buffer,PCHAR('\VarFileInfo\Translation'), buf2, buf2len) then
      begin
        FillChar(buffer3, SizeOf(buffer3), 0);
        Move(buf2^, buffer3, buf2len);
        intLocalization := integer(buffer3[2]) +
                           integer(buffer3[3]) * $100 +
                           integer(buffer3[0]) * $10000 +
                           integer(buffer3[1]) * $1000000;
        strLocalization := format('%x', [intLocalization]);
        while length(strLocalization) < 8 do strLocalization := '0' + strLocalization;
        lstFileInfo.Add('Localization=' + strLocalization);
        for i :=1 to InfoNum do
          if VerQueryValue(Buffer,PChar('StringFileInfo\' + strLocalization + '\'+
                                     InfoStr[i]),Pointer(Value),Len) then
            lstFileInfo.Add('L_' + InfoStr[i]+'=' + Value);
      end;
    end;
  finally
    FreeMem(buffer);
  end;
end;

function StabGetFileTime(const FileName: String; ComparisonType: TTimeOfWhat): TFileTime;
var
  FileTime, LocalFileTime: TFileTime;
  hFile                  : THandle;
begin
  // initialize TFileTime record in case of error
  Result.dwLowDateTime := 0;
  Result.dwHighDateTime := 0;
  hFile := FileOpen(FileName, fmShareDenyNone);
  try
    if hFile <> 0 then
    begin
      case ComparisonType of
        ftCreationTime   : Windows.GetFileTime(hFile, @FileTime, nil, nil);
        ftLastAccessTime : Windows.GetFileTime(hFile, nil, @FileTime, nil);
        ftLastWriteTime  : Windows.GetFileTime(hFile, nil, nil, @FileTime);
      end; // case FileTimeOf
      // Change the file time to local time
      FileTimeToLocalFileTime( FileTime, LocalFileTime );
      Result := LocalFileTime;
    end; // if hFile <> 0
  finally
    FileClose( hFile );
  end; // try
end;

function StabCompareFileTime(const FileNameOne, FileNameTwo: String;
                             ComparisonType: TTimeOfWhat): TFileTimeComparision;
var
  FileOneFileTime: TFileTime;
  FileTwoFileTime: TFileTime;
begin
  Result := ftError;

  FileOneFileTime := StabGetFileTime(FileNameOne, ComparisonType);
  FileTwoFileTime := StabGetFileTime(FileNameTwo, ComparisonType);

  case Windows.CompareFileTime(FileOneFileTime, FileTwoFileTime) of
    -1: Result := ftFileOneIsOlder;
     0: Result := ftFileTimesAreEqual;
     1: Result := ftFileTwoIsOlder;
  end;
end;

function StabGetFileTimeToDateTime(FileTime: TFileTime): TDateTime;
var
  SystemTime : TSystemTime;
begin
  Result := EncodeDate(9999,12,31);
  if FileTimeToSystemTime( FileTime, SystemTime ) then
    // Convert to TDateTime and return
    Result := SystemTimeToDateTime( SystemTime );
end;

end.
